@@ -12,6 +12,7 @@
 #include <QApplication>
 #include <QMouseEvent>
 #include <QFontMetrics>
+#include <QCalendarWidget>
 #include <QContextMenuEvent>
 #include <QMenu>
 #include <QContextMenuEvent>
@@ -29,6 +30,7 @@
 #include <QDate>
 #include <QDesktopServices>
 #include <QToolTip>
+#include <QPlainTextEdit>
 #include <QDebug>
 
 #include <libqalculate/qalculate.h>
@@ -175,6 +177,9 @@ HistoryView::HistoryView(QWidget *parent) : QTextEdit(parent) {
 	findAction = new QAction(tr("Search…"), this);
 	addAction(findAction);
 	connect(findAction, SIGNAL(triggered()), this, SLOT(editFind()));
+	findDateAction = new QAction(tr("Search by Date…"), this);
+	addAction(findDateAction);
+	connect(findDateAction, SIGNAL(triggered()), this, SLOT(editFindDate()));
 }
 HistoryView::~HistoryView() {}
 
@@ -408,10 +413,15 @@ void HistoryView::clearTemporary() {
 
 bool HistoryView::testTemporaryResultLength(const std::string &str) {
 	QFontMetrics fm(font());
-	return fm.boundingRect(QString("#9999") + "= " + unhtmlize(QString::fromStdString(str))).width() < width() * 1.65;
+	return fm.boundingRect(QStringLiteral("#9999") + "= " + unhtmlize(QString::fromStdString(str))).width() < width() * 1.65;
 }
 
-void HistoryView::addResult(std::vector<std::string> values, std::string expression, bool pexact, std::string parse, int exact, bool dual_approx, const QString &image, bool *implicit_warning, int initial_load, size_t index, bool temporary) {
+int HistoryView::maxTemporaryCharacters() {
+	QFontMetrics fm(font());
+	return (width() * 1.65 / fm.averageCharWidth()) - 7;
+}
+
+void HistoryView::addResult(std::vector<std::string> values, std::string expression, bool pexact, std::string parse, int exact, bool dual_approx, const QString &image, bool *implicit_warning, int initial_load, size_t index, bool temporary, const std::string &tmp_value) {
 	if(temporary && !previous_temporary) {
 		previous_cursor2 = previous_cursor;
 	}
@@ -430,6 +440,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 					mstr.replace(">=", SIGN_GREATER_OR_EQUAL);
 					mstr.replace("<=", SIGN_LESS_OR_EQUAL);
 					mstr.replace("!=", SIGN_NOT_EQUAL);
+					mstr.replace("\n", "<br>");
 				}
 				MessageType mtype = CALCULATOR->message()->type();
 				if(temporary) {
@@ -441,8 +452,8 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 						serror += mstr;
 					}
 				} else {
-					serror += "<tr><td class=\"message\" colspan=\"2\" style=\"text-align:left; font-size:normal";
 					if(mtype == MESSAGE_ERROR || mtype == MESSAGE_WARNING) {
+						serror += "<tr><td class=\"message\" colspan=\"2\" style=\"text-align:left; font-size:normal";
 						serror += "; color: ";
 						if(mtype == MESSAGE_ERROR) {
 							if(settings->color == 2) serror += "#FFAAAA";
@@ -453,11 +464,13 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 						}
 						serror += "";
 						if(CALCULATOR->message()->stage() == MESSAGE_STAGE_PARSING) b_parse_error = true;
+						serror += "\">";
+						if(!mstr.startsWith("-")) serror += "- ";
+						serror += mstr.toHtmlEscaped();
+						serror += "</td></tr>";
+					} else {
+						values.insert(values.begin(), std::string("#") + mstr.toStdString());
 					}
-					serror += "\">";
-					if(!mstr.startsWith("-")) serror += "- ";
-					serror += mstr.toHtmlEscaped();
-					if(!temporary) serror += "</td></tr>";
 				}
 			}
 		} while(CALCULATOR->nextMessage());
@@ -465,12 +478,16 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 	}
 	PASTE_H
 	QString str;
+	bool comment = false;
 	if(!expression.empty() || !parse.empty()) {
-		if(initial_load && index != settings->v_expression.size() - 1) str += QString("<tr><td colspan=\"2\" style=\"padding-bottom: %1 px; padding-top: %3px; border-top: 1px dashed %2; text-align:left\">").arg(paste_h / 4).arg(text_color.name()).arg(paste_h / 2);
-		else str += QString("<tr><td colspan=\"2\" style=\"padding-bottom: %1 px; padding-top: 0px; border-top: 0px none %2; text-align:left\">").arg(paste_h / 4).arg(text_color.name());
+		comment = !temporary && expression.empty() && parse == "#";
+		if(initial_load && index != settings->v_expression.size() - 1) str += QStringLiteral("<tr><td colspan=\"2\" style=\"padding-bottom: %1 px; padding-top: %3px; border-top: 1px dashed %2; text-align:left").arg(comment ? 0 : paste_h / 4).arg(text_color.name()).arg(paste_h / 2);
+		else str += QStringLiteral("<tr><td colspan=\"2\" style=\"padding-bottom: %1 px; padding-top: 0px; border-top: 0px none %2; text-align:left").arg(comment ? 0 : paste_h / 4).arg(text_color.name());
+		if(comment) str += "; line-height: 1%";
+		str += "\">";
 		if(temporary) {
 			parse_tmp = parse;
-			result_tmp = (values.empty() ? "" : values[0]);
+			result_tmp = tmp_value;
 		} else if(!initial_load) {
 			settings->v_expression.push_back(expression);
 			settings->v_parse.push_back(parse);
@@ -486,20 +503,22 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 			settings->v_result.push_back(values);
 			settings->v_exact.push_back(std::vector<int>());
 			settings->v_value.push_back(std::vector<size_t>());
+			size_t n = 0;
 			for(size_t i = 0; i < values.size(); i++) {
 				settings->v_exact[settings->v_exact.size() - 1].push_back(exact || i < values.size() - 1);
-				settings->v_value[settings->v_value.size() - 1].push_back(dual_approx && i == 0 ? settings->history_answer.size() - 1 : settings->history_answer.size());
+				settings->v_value[settings->v_value.size() - 1].push_back(dual_approx && n == 0 ? settings->history_answer.size() - 1 : settings->history_answer.size());
+				if(values[i].empty() || values[i][0] != '#') n++;
 			}
 		}
 		gsub("</i>", "<img src=\"data://img1px.png\" width=\"1\"/></i>", parse);
-		if(!temporary && !expression.empty() && (settings->history_expression_type > 0 || parse.empty())) {
+		if(!comment && !temporary && !expression.empty() && (settings->history_expression_type > 0 || parse.empty())) {
 			if(!parse.empty() && settings->history_expression_type > 1 && parse != expression) {
-				str += QString("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QStringLiteral("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
 				str += QString::fromStdString(expression).toHtmlEscaped();
 				str += "</a>";
 				if(settings->color == 2) str += "&nbsp; <i style=\"color: #AAAAAA\">";
 				else str += "&nbsp; <i style=\"color: #666666\">";
-				str += QString("<a name=\"a%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QStringLiteral("<a name=\"a%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
 				if(pexact) str += "= ";
 				else str += SIGN_ALMOST_EQUAL " ";
 				if(!settings->colorize_result) str += QString::fromStdString(uncolorize(parse, false));
@@ -507,13 +526,13 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 				str += "</a>";
 				str += "</i>";
 			} else {
-				str += QString("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QStringLiteral("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
 				str += QString::fromStdString(expression).toHtmlEscaped();
 				str += "</a>";
 			}
-		} else {
+		} else if(!comment) {
 			if(temporary) str += "<a name=\"TP\" style=\"text-decoration: none\">";
-			else str += QString("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+			else str += QStringLiteral("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
 			if(!pexact) str += SIGN_ALMOST_EQUAL " ";
 			if(!settings->colorize_result) str += QString::fromStdString(uncolorize(parse, false));
 			else str += QString::fromStdString(parse);
@@ -522,7 +541,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 				str += "&nbsp;&nbsp;&nbsp; ";
 				if(settings->color == 2) str += "<i style=\"color: #AAAAAA\">";
 				else str += "<i style=\"color: #666666\">";
-				str += QString("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QStringLiteral("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
 				str += QString::fromStdString(replace_parse_colors(expression));
 				str += "</a>";
 				str += "</i>";
@@ -539,7 +558,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		}
 		if(temporary) {
 			temporary_error = serror;
-			if(!serror.isEmpty()) str += QString("<img src=\"data://img1px.png\" width=\"2\"/><img valign=\"top\" src=\"%1\" height=\"%2\" alt=\"temporary_error\"/>").arg(":/icons/actions/scalable/warning.svg").arg(fm.ascent());
+			if(!serror.isEmpty()) str += QStringLiteral("<img src=\"data://img1px.png\" width=\"2\"/><img valign=\"top\" src=\"%1\" height=\"%2\" alt=\"temporary_error\"/>").arg(":/icons/actions/scalable/warning.svg").arg(fm.ascent());
 		}
 		str += "</td></tr>";
 	} else if(!initial_load && !settings->v_result.empty()) {
@@ -563,10 +582,24 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 	if(temporary && values.empty()) {
 		values.push_back("");
 	}
+	size_t n = 0;
 	for(size_t i = 0; i < values.size(); i++) {
 		size_t i_answer = 0;
 		if(initial_load) i_answer = settings->v_value[index][i];
-		else if(!temporary) i_answer = dual_approx && i == 0 ? settings->history_answer.size() - 1 : settings->history_answer.size();
+		else if(!temporary) i_answer = dual_approx && n == 0 ? settings->history_answer.size() - 1 : settings->history_answer.size();
+		if(!values[i].empty() && values[i][0] == '#') {
+			str += "<tr><td colspan=\"2\" style=\"text-align:left";
+			if(!comment && (initial_load == 1 || (initial_load > 1 && n > 0))) str += "; font-size:small";
+			else if(!comment) str += "; font-size:normal";
+			if((!expression.empty() || !parse.empty()) && i == values.size() - 1) str += QStringLiteral("; padding-bottom: %1 px").arg(paste_h / 4);
+			str += "\">";
+			if(i_answer == 0) str += QStringLiteral("<a name=\"%1:%2\" style=\"text-decoration: none\">").arg(initial_load ? index : settings->v_expression.size() - 1).arg(initial_load ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
+			else str += QStringLiteral("<a name=\"c%1:%2:%3\" style=\"text-decoration: none\">").arg(i_answer).arg(initial_load ? index : settings->v_expression.size() - 1).arg(initial_load == 1 ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
+			if(values[i].size() == 1 || values[i][1] != '-') str += "- ";
+			str += QString::fromStdString(values[i]).mid(1);
+			str += "</a></td></tr>";
+			continue;
+		}
 		QFontMetrics fm(font());
 		int w_number = fm.boundingRect("#9999").width();
 		int w = 0;
@@ -580,27 +613,26 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 			}
 			QString sref;
 			if(settings->color == 2) {
-				sref = QString("<a href=\"#%1:%2:%3\" style=\"text-decoration: none; text-align:left; color: #AAAAAA\">#%1</a>").arg(i_answer).arg(initial_load ? (int) index : settings->v_expression.size() - 1).arg(initial_load ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
+				sref = QStringLiteral("<a href=\"#%1:%2:%3\" style=\"text-decoration: none; text-align:left; color: #AAAAAA\">#%1</a>").arg(i_answer).arg(initial_load ? (int) index : settings->v_expression.size() - 1).arg(initial_load == 1 ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
 			} else {
-				sref = QString("<a href=\"#%1:%2:%3\" style=\"text-decoration: none; text-align:left; color: #585858\">#%1</a>").arg(i_answer).arg(initial_load ? (int) index : settings->v_expression.size() - 1).arg(initial_load ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
+				sref = QStringLiteral("<a href=\"#%1:%2:%3\" style=\"text-decoration: none; text-align:left; color: #585858\">#%1</a>").arg(i_answer).arg(initial_load ? (int) index : settings->v_expression.size() - 1).arg(initial_load == 1 ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
 			}
 			bool b = true;
 			if(initial_load) {
 				for(size_t i2 = 0; i2 < i; i2++) {
-					if(settings->v_value[index][i2] == i_answer) {
+					if(settings->v_value[index][i2] == i_answer && (values[i2].empty() || values[i2][0] != '#')) {
 						b = false;
 					}
 				}
 			}
 			if(b) str += sref;
-			if((initial_load > 1 && i == 0) || (!initial_load && (!dual_approx || i == values.size() - 1))) {
+			if((initial_load > 1 && n == 0) || (!initial_load && (!dual_approx || i == values.size() - 1))) {
 				last_ans = i_answer;
 				last_ref = sref;
 			}
 		}
 		str += "</td><td style=\"text-align:right";
-		if(initial_load > 1 && i > 0 && i_answer <= i_answer_pre) initial_load = 1;
-		if(initial_load == 1 || w * 1.5 + w_number > width() || !settings->format_result) {
+		if(initial_load == 1 || (initial_load > 1 && n > 0 && i_answer <= i_answer_pre) || w * 1.5 + w_number > width() || !settings->format_result) {
 			gsub("</i>", "<img src=\"data://img1px.png\" width=\"1\"/></i>", values[i]);
 		} else if(w * 1.85 + w_number > width()) {
 			str += "; font-size:large";
@@ -610,7 +642,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 			str += "; font-size:x-large";
 			gsub("</i>", "<img src=\"data://img1px.png\" width=\"2\"/></i>", values[i]);
 		}
-		if((!expression.empty() || !parse.empty()) && i == values.size() - 1) str += QString("; padding-bottom: %1 px").arg(paste_h / 4);
+		if((!expression.empty() || !parse.empty()) && i == values.size() - 1) str += QStringLiteral("; padding-bottom: %1 px").arg(paste_h / 4);
 		str += "\">";
 		int b_exact = 1;
 		if(initial_load) {
@@ -622,8 +654,8 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		if(temporary && !values[i].empty()) {
 			str += "<a name=\"TR\" style=\"text-decoration: none\">";
 		} else if(!temporary) {
-			if(i_answer == 0) str += QString("<a name=\"%1:%2\" style=\"text-decoration: none\">").arg(initial_load ? index : settings->v_expression.size() - 1).arg(initial_load ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
-			else str += QString("<a name=\"p%1:%2:%3\" style=\"text-decoration: none\">").arg(i_answer).arg(initial_load ? index : settings->v_expression.size() - 1).arg(initial_load ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
+			if(i_answer == 0) str += QStringLiteral("<a name=\"%1:%2\" style=\"text-decoration: none\">").arg(initial_load ? index : settings->v_expression.size() - 1).arg(initial_load ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
+			else str += QStringLiteral("<a name=\"p%1:%2:%3\" style=\"text-decoration: none\">").arg(i_answer).arg(initial_load ? index : settings->v_expression.size() - 1).arg(initial_load == 1 ? (int) i : settings->v_result[settings->v_result.size() - 1].size() - i - 1);
 		}
 		if(!temporary || !values[i].empty()) {
 			if(b_exact > 0) str += "= ";
@@ -632,7 +664,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		if(!settings->colorize_result) str += QString::fromStdString(uncolorize(values[i], false));
 		else str += QString::fromStdString(values[i]);
 		if(!image.isEmpty() && i == values.size() - 1 && w * 1.85 + w_number + fm.ascent() <= width()) {
-			str += QString("<img src=\"data://img1px.png\" width=\"2\"/><img valign=\"top\" src=\"%1\" height=\"%2\"").arg(image).arg(fm.ascent());
+			str += QStringLiteral("<img src=\"data://img1px.png\" width=\"2\"/><img valign=\"top\" src=\"%1\" height=\"%2\"").arg(image).arg(fm.ascent());
 			CALCULATOR->setExchangeRatesUsed(-100);
 			int i = CALCULATOR->exchangeRatesUsed();
 			CALCULATOR->setExchangeRatesUsed(-100);
@@ -658,6 +690,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		if(!temporary || !values[i].empty()) str += "</a>";
 		str += "</td></tr>";
 		i_answer_pre = i_answer;
+		n++;
 	}
 	str.replace("\n", "<br>");
 	int i = 0;
@@ -669,7 +702,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		last_ref2 = last_ref;
 	}
 	if(!initial_load) {
-		if(settings->format_result) {
+		if(settings->format_result && n > 0) {
 			previous_html.replace("font-size:normal", "font-size:small ");
 			previous_html.replace("width=\"2\"", "width=\"1\"");
 		}
@@ -678,7 +711,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 			if(i < 0) break;
 			int i2 = previous_html.indexOf(">", i);
 			previous_html.remove(i, i2 - i + 1);
-			if(i < i_pos_p) i_pos_p -= (i2 - i + 1);
+			if(i < i_pos_p && expression.empty() && parse.empty()) i_pos_p -= (i2 - i + 1);
 		}
 		QTextCursor cur = textCursor();
 		cur.setPosition(previous_cursor);
@@ -687,7 +720,7 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		cur.deleteChar();
 	}
 	if(expression.empty() && parse.empty()) {
-		if(!initial_load && settings->format_result) {
+		if(!initial_load && settings->format_result && n > 0) {
 			i = 0;
 			while(true) {
 				i = previous_html.indexOf("; font-size:x-large", i);
@@ -815,7 +848,9 @@ void HistoryView::mouseDoubleClickEvent(QMouseEvent *e) {
 		int i1 = -1, i2 = -1;
 		indexAtPos(e->pos(), &i1, &i2, NULL, &str);
 		if(i1 >= 0 && str.length() > 0) {
-			if(str[0] == 'p') {
+			if(str[0] == 'c') {
+				editComment(i1, i2);
+			} else if(str[0] == 'p') {
 				int index = str.indexOf(":");
 				if(index < 0) return;
 				str = str.mid(index + 1);
@@ -845,7 +880,10 @@ void HistoryView::mouseDoubleClickEvent(QMouseEvent *e) {
 				} else {
 					int i1 = str.left(index).toInt();
 					int i2 = str.mid(index + 1).toInt();
-					if(i1 >= 0 && (size_t) i1 < settings->v_result.size() && i2 >= 0 && (size_t) i2 < settings->v_result[i1].size()) emit insertTextRequested(settings->v_result[i1][i2]);
+					if(i1 >= 0 && (size_t) i1 < settings->v_result.size() && i2 >= 0 && (size_t) i2 < settings->v_result[i1].size()) {
+						if(!settings->v_result[i1][i2].empty() && settings->v_result[i1][i2][0] == '#') editComment(i1, i2);
+						else emit insertTextRequested(settings->v_result[i1][i2]);
+					}
 				}
 			}
 		}
@@ -974,10 +1012,19 @@ void HistoryView::editRemove() {
 	indexAtPos(context_pos, &i1, &i2);
 	if(i1 < 0 || i1 >= (int) settings->v_expression.size()) return;
 	if(i2 >= 0 && i2 < (int) settings->v_result[i1].size()) {
-		if(settings->v_result[i1].size() == 1) {
+		bool remove_expression = true;
+		for(size_t i = 0; i < settings->v_result[i1].size(); i++) {
+			if(i != (size_t) i2 && (settings->v_result[i1][i].empty() || settings->v_result[i1][i][0] != '#')) {
+				remove_expression = false;
+				break;
+			}
+		}
+		if(remove_expression) {
 			i2 = -1;
 		} else {
 			settings->v_result[i1].erase(settings->v_result[i1].begin() + i2);
+			settings->v_exact[i1].erase(settings->v_exact[i1].begin() + i2);
+			settings->v_value[i1].erase(settings->v_value[i1].begin() + i2);
 		}
 	}
 	if(i2 < 0) {
@@ -997,6 +1044,68 @@ void HistoryView::editRemove() {
 	reloadHistory();
 	verticalScrollBar()->setValue(vpos);
 }
+void HistoryView::editComment(int i1, int i2) {
+	bool b_edit = !settings->v_result[i1][i2].empty() && settings->v_result[i1][i2][0] == '#';
+	if(i2 > 0 && !settings->v_result[i1][i2 - 1].empty() && settings->v_result[i1][i2 - 1][0] == '#') {
+		i2--;
+		b_edit = true;
+	}
+	QDialog *dialog = new QDialog(this);
+	QVBoxLayout *box = new QVBoxLayout(dialog);
+	if(settings->always_on_top) dialog->setWindowFlags(dialog->windowFlags() | Qt::WindowStaysOnTopHint);
+	dialog->setWindowTitle(tr("Comment"));
+	QGridLayout *grid = new QGridLayout();
+	QPlainTextEdit *commentEdit = new QPlainTextEdit(this);
+	if(b_edit) {
+		QString str = QString::fromStdString(unhtmlize(settings->v_result[i1][i2].substr(1)));
+		commentEdit->setPlainText(str);
+		commentEdit->selectAll();
+	}
+	commentEdit->setFocus();
+	grid->addWidget(commentEdit, 0, 0);
+	box->addLayout(grid);
+	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Cancel | QDialogButtonBox::Ok, Qt::Horizontal, dialog);
+	buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
+	buttonBox->button(QDialogButtonBox::Cancel)->setAutoDefault(false);
+	box->addWidget(buttonBox);
+	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), dialog, SLOT(accept()));
+	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), dialog, SLOT(reject()));
+	if(dialog->exec() == QDialog::Accepted && (b_edit || !commentEdit->toPlainText().trimmed().isEmpty())) {
+		if(b_edit && commentEdit->toPlainText().trimmed().isEmpty()) {
+			editRemove();
+		} else {
+			QString str = "#" + commentEdit->toPlainText().trimmed().toHtmlEscaped();
+			str.replace("\n", "<br>");
+			if(b_edit) {
+				if(settings->v_result[i1][i2] != str.toStdString()) {
+					settings->v_result[i1][i2] = str.toStdString();
+					int vpos = verticalScrollBar()->value();
+					reloadHistory();
+					verticalScrollBar()->setValue(vpos);
+				}
+			} else if(!settings->history_answer.empty() && i1 == (int) settings->v_result.size() - 1 && i2 == 0) {
+				std::vector<std::string> values;
+				values.push_back(str.toStdString());
+				addResult(values, "", true, "", settings->v_exact[i1][i2]);
+			} else {
+				settings->v_result[i1].insert(settings->v_result[i1].begin() + i2, str.toStdString());
+				settings->v_exact[i1].insert(settings->v_exact[i1].begin() + i2, settings->v_exact[i1][i2]);
+				settings->v_value[i1].insert(settings->v_value[i1].begin() + i2, settings->v_value[i1][i2]);
+				int vpos = verticalScrollBar()->value();
+				reloadHistory();
+				verticalScrollBar()->setValue(vpos);
+			}
+		}
+	}
+	dialog->deleteLater();
+}
+void HistoryView::editComment() {
+	int i1 = -1, i2 = -1;
+	indexAtPos(context_pos, &i1, &i2);
+	if(i1 < 0 || i1 >= (int) settings->v_expression.size()) return;
+	if(i2 < 0 || i2 >= (int) settings->v_result[i1].size()) i2 = 0;
+	editComment(i1, i2);
+}
 void HistoryView::editProtect() {
 	int i1 = -1, i2 = -1;
 	indexAtPos(context_pos, &i1, &i2);
@@ -1038,7 +1147,7 @@ void HistoryView::indexAtPos(const QPoint &pos, int *expression_index, int *resu
 	if(anchorstr) *anchorstr = sref;
 	if(sref.isEmpty() || sref == "TR" || sref == "TP") return;
 	int i = sref.indexOf(":");
-	if(sref[0] == '#' || sref[0] == 'p') {
+	if(sref[0] == '#' || sref[0] == 'p' || sref[0] == 'c') {
 		if(i < 0) return;
 		int i2 = sref.indexOf(":", i + 1);
 		if(i2 >= 0) {
@@ -1083,9 +1192,11 @@ void HistoryView::contextMenuEvent(QContextMenuEvent *e) {
 		selectAllAction->setShortcut(QKeySequence::SelectAll);
 		selectAllAction->setShortcutContext(Qt::WidgetShortcut);
 		cmenu->addAction(findAction);
+		cmenu->addAction(findDateAction);
 		cmenu->addSeparator();
 		protectAction = cmenu->addAction(tr("Protect"), this, SLOT(editProtect()));
 		protectAction->setCheckable(true);
+		commentAction = cmenu->addAction(tr("Add Comment…"), this, SLOT(editComment()));
 		movetotopAction = cmenu->addAction(tr("Move to Top"), this, SLOT(editMoveToTop()));
 		cmenu->addSeparator();
 		delAction = cmenu->addAction(tr("Remove"), this, SLOT(editRemove()));
@@ -1105,7 +1216,9 @@ void HistoryView::contextMenuEvent(QContextMenuEvent *e) {
 	selectAllAction->setEnabled(!document()->isEmpty());
 	protectAction->setChecked(i1 >= 0 && i1 < (int) settings->v_protected.size() && settings->v_protected[i1]);
 	if((i1 >= 0 || b_tmp) && e->reason() == QContextMenuEvent::Mouse && !textCursor().hasSelection()) {
-		insertValueAction->setEnabled(i3 >= 0);
+		if(i2 < 0) i2 = 0;
+		bool b_edit = (!astr.isEmpty() && astr[0] == 'c') || (!b_tmp && i2 < (int) settings->v_result[i1].size() && ((!settings->v_result[i1][i2].empty() && settings->v_result[i1][i2][0] == '#') || (i2 > 0 && !settings->v_result[i1][i2 - 1].empty() && settings->v_result[i1][i2 - 1][0] == '#')));
+		insertValueAction->setEnabled(i3 >= 0 && !b_edit);
 		insertTextAction->setEnabled(true);
 		copyAction->setEnabled(true);
 		copyFormattedAction->setEnabled(true);
@@ -1113,6 +1226,15 @@ void HistoryView::contextMenuEvent(QContextMenuEvent *e) {
 		delAction->setEnabled(i1 >= 0);
 		protectAction->setEnabled(i1 >= 0);
 		movetotopAction->setEnabled(i1 >= 0 && !settings->v_expression.empty());
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+		if(i1 >= 0 && !settings->v_time.empty() && settings->v_time[i1] != 0) findDateAction->setText(tr("Search by Date…") + " (" + QDateTime::fromSecsSinceEpoch(settings->v_time[i1]).date().toString(Qt::ISODate) + ")");
+#else
+		if(i1 >= 0 && !settings->v_time.empty() && settings->v_time[i1] != 0) findDateAction->setText(tr("Search by Date…") + " (" + QDateTime::fromMSecsSinceEpoch(settings->v_time[i1] * 1000).date().toString(Qt::ISODate) + ")");
+#endif
+		else findDateAction->setText(tr("Search by Date…"));
+		commentAction->setEnabled(i1 >= 0);
+		if(b_edit) commentAction->setText(tr("Edit Comment…"));
+		else commentAction->setText(tr("Add Comment…"));
 	} else {
 		copyAction->setEnabled(textCursor().hasSelection());
 		copyFormattedAction->setEnabled(textCursor().hasSelection());
@@ -1122,7 +1244,12 @@ void HistoryView::contextMenuEvent(QContextMenuEvent *e) {
 		movetotopAction->setEnabled(false);
 		insertValueAction->setEnabled(false);
 		insertTextAction->setEnabled(false);
+		findDateAction->setText(tr("Search by Date…"));
+		commentAction->setEnabled(false);
+		commentAction->setText(tr("Add Comment…"));
 	}
+	findAction->setEnabled(!settings->v_expression.empty());
+	findDateAction->setEnabled(!settings->v_expression.empty());
 	clearAction->setEnabled(!settings->v_expression.empty());
 	cmenu->popup(e->globalPos());
 }
@@ -1182,6 +1309,50 @@ void HistoryView::editFind() {
 	searchDialog->show();
 }
 
+void HistoryView::editFindDate() {
+	QDialog *dialog = new QDialog(this, Qt::Popup);
+	if(settings->always_on_top) dialog->setWindowFlags(dialog->windowFlags() | Qt::WindowStaysOnTopHint);
+	QVBoxLayout *box = new QVBoxLayout(dialog);
+	box->setContentsMargins(0, 0, 0, 0);
+	QCalendarWidget *w = new QCalendarWidget(dialog);
+	if(!settings->v_time.empty() && settings->v_time[settings->v_time.size() - 1] > 0) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+		w->setMaximumDate(QDateTime::fromSecsSinceEpoch(settings->v_time[settings->v_time.size() - 1]).date());
+#else
+		w->setMaximumDate(QDateTime::fromMSecsSinceEpoch(settings->v_time[settings->v_time.size() - 1] * 1000).date());
+#endif
+		for(size_t i = settings->v_time.size() - 1; ; i--) {
+			if(i == 0 || settings->v_time[i] == 0) {
+				if(settings->v_time[i] == 0) i++;
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+				w->setMinimumDate(QDateTime::fromSecsSinceEpoch(settings->v_time[i]).date());
+#else
+				w->setMinimumDate(QDateTime::fromMSecsSinceEpoch(settings->v_time[i] * 1000).date());
+#endif
+				break;
+			}
+		}
+	}
+	box->addWidget(w);
+	connect(w, SIGNAL(clicked(QDate)), dialog, SLOT(accept()));
+	connect(w, SIGNAL(activated(QDate)), dialog, SLOT(accept()));
+	if(dialog->exec() == QDialog::Accepted) {
+		QDate date = w->selectedDate();
+		for(size_t i = settings->v_time.size(); i > 0; i--) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+			if(settings->v_time[i - 1] != 0 && QDateTime::fromSecsSinceEpoch(settings->v_time[i - 1]).date() <= date) {
+#else
+			if(settings->v_time[i - 1] != 0 && QDateTime::fromMSecsSinceEpoch(settings->v_time[i - 1] * 1000).date() <= date) {
+#endif
+				if(settings->history_expression_type > 1) scrollToAnchor("e" + QString::number(i - 1));
+				else scrollToAnchor(QString::number(i - 1));
+				break;
+			}
+		}
+	}
+	dialog->deleteLater();
+}
+
 void remove_separator(std::string &copy_text) {
 	for(size_t i = ((CALCULATOR->local_digit_group_separator.empty() || CALCULATOR->local_digit_group_separator == " " || CALCULATOR->local_digit_group_separator == settings->printops.decimalpoint()) ? 1 : 0); i < 4; i++) {
 		std::string str_sep;
@@ -1396,11 +1567,14 @@ void HistoryView::editCopy(int ascii) {
 			if(b_temp || ((size_t) i1 < settings->v_result.size() && (size_t) i2 < settings->v_result[i1].size())) {
 				if(ascii) {
 					std::string str = (b_temp ? result_tmp : settings->v_result[i1][i2]);
+					if(!str.empty() && str[0] == '#') str.erase(0, 1);
 					if(settings->copy_ascii_without_units) {
 						size_t i = str.find("<span style=\"color:#008000\">");
 						if(i == std::string::npos) i = str.find("<span style=\"color:#BBFFBB\">");
-						if(i != std::string::npos && str.find("</span>", i) == str.length() - 7) {
-							str = str.substr(0, i);
+						if(i != std::string::npos) {
+							size_t iend = str.find("</span>", i);
+							if(i == 0) str = str.substr(iend + 7);
+							else if(iend == str.length() - 7) str = str.substr(0, i);
 						}
 					}
 					QApplication::clipboard()->setText(QString::fromStdString(unformat(unhtmlize(str, true))).trimmed());
@@ -1409,6 +1583,9 @@ void HistoryView::editCopy(int ascii) {
 					if(b_temp) {
 						qm->setHtml(QString::fromStdString(uncolorize(result_tmp)));
 						qm->setText(QString::fromStdString(unhtmlize((result_tmp))));
+					} else if(!settings->v_result[i1][i2].empty() && settings->v_result[i1][i2][0] == '#') {
+						qm->setHtml(QString::fromStdString(uncolorize(settings->v_result[i1][i2].substr(1))));
+						qm->setText(QString::fromStdString(unhtmlize((settings->v_result[i1][i2].substr(1)))));
 					} else {
 						qm->setHtml(QString::fromStdString(uncolorize(settings->v_result[i1][i2])));
 						qm->setText(QString::fromStdString(unhtmlize((settings->v_result[i1][i2]))));
@@ -1440,7 +1617,10 @@ void HistoryView::editInsertText() {
 	} else if(i1 < 0 && astr == "TP") {
 		emit insertTextRequested(parse_tmp);
 	} else if(i2 >= 0) {
-		if(i1 >= 0 && (size_t) i1 < settings->v_result.size() && (size_t) i2 < settings->v_result[i1].size()) emit insertTextRequested(settings->v_result[i1][i2]);
+		if(i1 >= 0 && (size_t) i1 < settings->v_result.size() && (size_t) i2 < settings->v_result[i1].size()) {
+			if(!settings->v_result[i1][i2].empty() && settings->v_result[i1][i2][0] == '#') emit insertTextRequested(settings->v_result[i1][i2].substr(1));
+			else emit insertTextRequested(settings->v_result[i1][i2]);
+		}
 	} else {
 		if(i1 >= 0 && (size_t) i1 < settings->v_expression.size()) emit insertTextRequested(settings->v_expression[i1].empty() ? settings->v_parse[i1] : settings->v_expression[i1]);
 	}
