@@ -35,24 +35,32 @@ class HistoryView : public QTextEdit {
 		QAction *findAction;
 		QDialog *searchDialog;
 
-		void addResult(std::vector<std::string> values, std::string expression = "", bool pexact = true, std::string parse = "", int exact = 1, bool dual_approx = false, const QString &image = QString(), bool *implicit_warning = NULL, bool initial_load = false, size_t index = 0);
+		void addResult(std::vector<std::string> values, std::string expression = "", bool pexact = true, std::string parse = "", int exact = 1, bool dual_approx = false, const QString &image = QString(), bool *implicit_warning = NULL, int initial_load = 0, size_t index = 0, bool temporary = false, const std::string &tmp_value = "");
+		void clearTemporary();
 		void addMessages();
-		void loadInitial();
+		void loadInitial(bool reload = false);
 		void indexAtPos(const QPoint &pos, int *expression_index, int *result_index, int *value_index = NULL, QString *anchorstr = NULL);
-		void replaceColors(QString&);
+		void replaceColors(QString&, QColor prev_text_color = QColor());
+		void setMenuAndToolbarItems(QMenu*, QMenu*, QAction*);
+		void updateMinimumHeight();
+		bool testTemporaryResultLength(const std::string&);
+		int maxTemporaryCharacters();
 
 	protected:
 
-		QString s_text;
-		int i_pos;
+		QString s_text, previous_html, previous_html2, temporary_error;
+		std::string parse_tmp, result_tmp;
+		int i_pos, i_pos2, i_pos_p, i_pos_p2, previous_cursor, previous_cursor2, previous_temporary;
 		int has_lock_symbol;
-		QMenu *cmenu;
-		QAction *insertTextAction, *insertValueAction, *copyAction, *copyFormattedAction, *copyAsciiAction, *selectAllAction, *delAction, *clearAction, *protectAction, *movetotopAction;
-		QColor prev_color;
+		QMenu *cmenu, *fileMenu, *modeMenu;
+		QAction *insertTextAction, *insertValueAction, *copyAction, *copyFormattedAction, *copyAsciiAction, *selectAllAction, *delAction, *clearAction, *protectAction, *movetotopAction, *tbAction, *fileSeparator, *findDateAction, *commentAction;
+		QColor text_color;
+		QRect prev_fonti;
 		QPoint context_pos;
 		QLineEdit *searchEdit;
-		size_t last_ans;
-		QString last_ref;
+		size_t last_ans, last_ans2;
+		QString last_ref, last_ref2;
+		bool initial_loaded;
 
 		void mouseDoubleClickEvent(QMouseEvent *e) override;
 		void mouseReleaseEvent(QMouseEvent *e) override;
@@ -61,6 +69,7 @@ class HistoryView : public QTextEdit {
 		void keyPressEvent(QKeyEvent *e) override;
 		void inputMethodEvent(QInputMethodEvent*) override;
 		void changeEvent(QEvent*) override;
+		void editComment(int, int);
 
 	protected slots:
 
@@ -77,12 +86,16 @@ class HistoryView : public QTextEdit {
 		void editRemove();
 		void editProtect();
 		void editFind();
+		void editFindDate();
 		void editMoveToTop();
+		void editComment();
+		void reloadHistory();
 
 	signals:
 
 		void insertTextRequested(std::string);
 		void insertValueRequested(int);
+		void historyReloaded();
 
 };
 
