@@ -53,6 +53,7 @@ void remove_spaces(std::string &str);
 bool contains_plot_or_save(const std::string &str);
 void base_from_string(std::string str, int &base, Number &nbase, bool input_base = false);
 long int get_fixed_denominator_qt(const std::string &str, int &to_fraction, const QString &localized_fraction, bool qalc_command = false);
+bool is_equation_solutions(const MathStructure &m);
 
 enum {
 	TITLE_APP,
@@ -163,10 +164,11 @@ typedef enum {
 	SHORTCUT_TYPE_MINMAX_DECIMALS,
 	SHORTCUT_TYPE_FUNCTIONS_MENU,
 	SHORTCUT_TYPE_UNITS_MENU,
-	SHORTCUT_TYPE_VARIABLES_MENU
+	SHORTCUT_TYPE_VARIABLES_MENU,
+	SHORTCUT_TYPE_TOGGLE_FRACTION
 } shortcut_type;
 
-#define LAST_SHORTCUT_TYPE SHORTCUT_TYPE_VARIABLES_MENU
+#define LAST_SHORTCUT_TYPE SHORTCUT_TYPE_TOGGLE_FRACTION
 
 #define SHORTCUT_REQUIRES_VALUE(x) (x == SHORTCUT_TYPE_FUNCTION || x == SHORTCUT_TYPE_FUNCTION_WITH_DIALOG || x == SHORTCUT_TYPE_UNIT || x == SHORTCUT_TYPE_VARIABLE || x == SHORTCUT_TYPE_TEXT || x == SHORTCUT_TYPE_OPERATOR || x == SHORTCUT_TYPE_CONVERT_TO || x == SHORTCUT_TYPE_TO_NUMBER_BASE || x == SHORTCUT_TYPE_INPUT_BASE || x == SHORTCUT_TYPE_OUTPUT_BASE || x == SHORTCUT_TYPE_PRECISION || x == SHORTCUT_TYPE_MAX_DECIMALS || x == SHORTCUT_TYPE_MIN_DECIMALS || x == SHORTCUT_TYPE_MINMAX_DECIMALS)
 
@@ -266,7 +268,7 @@ class QalculateQtSettings : QObject {
 		bool save_defs_on_exit, save_mode_on_exit, clear_history_on_exit;
 		int max_history_lines;
 		bool rpn_shown;
-		bool auto_calculate, status_in_history;
+		bool auto_calculate, status_in_history, autocalc_selection;
 		int history_expression_type;
 		bool copy_ascii, copy_ascii_without_units;
 		bool close_with_esc;
@@ -281,6 +283,7 @@ class QalculateQtSettings : QObject {
 		QByteArray units_geometry, units_vsplitter_state, units_hsplitter_state;
 		QByteArray variables_geometry, variables_vsplitter_state, variables_hsplitter_state;
 		QByteArray datasets_geometry, datasets_vsplitter_state, datasets_hsplitter_state;
+		bool disable_cursor_blinking;
 		bool wayland_platform;
 
 		int preserve_history_height;
