@@ -104,7 +104,7 @@ class QalculateWindow : public QMainWindow {
 		QToolBar *tb;
 		QToolButton *menuAction_t, *modeAction_t, *keypadAction_t, *storeAction_t, *functionsAction_t, *unitsAction_t, *toAction_t;
 		QMenu *tmenu, *basesMenu;
-		QAction *plotAction_t, *basesAction, *customOutputBaseAction, *customInputBaseAction, *newVariableAction, *newFunctionAction, *variablesAction, *functionsAction, *unitsAction, *datasetsAction, *plotAction, *fpAction, *calendarsAction, *percentageAction, *periodicTableAction, *exratesAction, *quitAction, *helpAction, *keypadAction, *rpnAction, *chainAction, *gKeypadAction, *pKeypadAction, *xKeypadAction, *cKeypadAction, *hideNumpadAction, *resetKeypadPositionAction, *radAction, *degAction, *graAction, *normalAction, *sciAction, *engAction, *simpleAction, *copyBasesAction, *tbAction;
+		QAction *plotAction_t, *basesAction, *customOutputBaseAction, *customInputBaseAction, *newVariableAction, *newFunctionAction, *variablesAction, *functionsAction, *unitsAction, *datasetsAction, *plotAction, *fpAction, *calendarsAction, *percentageAction, *periodicTableAction, *exratesAction, *quitAction, *helpAction, *keypadAction, *rpnAction, *chainAction, *gKeypadAction, *pKeypadAction, *xKeypadAction, *cKeypadAction, *nKeypadAction, *showNumpadAction, *resetKeypadPositionAction, *radAction, *degAction, *graAction, *normalAction, *sciAction, *engAction, *simpleAction, *copyBasesAction, *tbAction;
 		QMenu *recentVariablesMenu, *favouriteVariablesMenu, *variablesMenu, *functionsMenu, *recentFunctionsMenu, *favouriteFunctionsMenu, *unitsMenu, *recentUnitsMenu, *favouriteUnitsMenu, *angleMenu, *toMenu;
 		QAction *assumptionTypeActions[5], *assumptionSignActions[6];
 		QMenu *recentWSMenu;
@@ -121,8 +121,8 @@ class QalculateWindow : public QMainWindow {
 		bool workspace_changed;
 		bool init_in_progress;
 
-		void calculateExpression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0, std::string execute_str = std::string(), std::string str = std::string(), bool check_exrates = true);
-		void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, std::string transformation = "", bool do_stack = false, size_t stack_index = 0, bool register_moved = false, bool supress_dialog = false);
+		void calculateExpression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0, std::string execute_str = std::string(), std::string str = std::string(), bool check_exrates = true, bool calculate_selection = false);
+		void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, std::string transformation = "", bool do_stack = false, size_t stack_index = 0, bool register_moved = false, bool supress_dialog = false, bool calculate_selection = false);
 		void executeCommand(int command_type, bool show_result = true, std::string ceu_str = "", Unit *u = NULL, int run = 1);
 		void changeEvent(QEvent *e) override;
 		bool askTC(MathStructure&);
@@ -183,7 +183,7 @@ class QalculateWindow : public QMainWindow {
 		void onToActivated(bool = true);
 		void onStoreActivated();
 		void keypadTypeActivated();
-		void hideNumpad(bool);
+		void showNumpad(bool);
 		void showSeparateKeypadMenuButtons(bool);
 		void resetKeypadPosition();
 		void onEMHTimeout();
@@ -257,12 +257,16 @@ class QalculateWindow : public QMainWindow {
 		void registerChanged(int);
 		void calculateRPN(int);
 		void approximateResult();
+		void toggleResultFraction();
 		void onExpressionStatusModeChanged(bool = true);
 		void functionActivated();
 		void unitActivated();
 		void prefixActivated();
 		void variableActivated();
 		void updateAngleUnitsMenu();
+		void initializeFunctionsMenu();
+		void initializeVariablesMenu();
+		void initializeUnitsMenu();
 		void updateFunctionsMenu();
 		void useFunctionDialog(bool);
 		void showAllFunctions(bool);
@@ -314,6 +318,7 @@ class QalculateWindow : public QMainWindow {
 		void socketReadyRead();
 		void calculate();
 		void calculate(const QString&);
+		void calculateSelection();
 		void onActivateRequested(const QStringList&, const QString&);
 		void abort();
 		void abortCommand();
@@ -322,6 +327,7 @@ class QalculateWindow : public QMainWindow {
 		void expressionCalculationUpdated(int delay = 0);
 		void resultFormatUpdated(int delay = 0);
 		void resultDisplayUpdated();
+		void expressionObjectsUpdated();
 		void expressionFormatUpdated(bool);
 		void insertFunction(MathFunction*, QWidget* = NULL);
 		void newVariable();
@@ -334,11 +340,13 @@ class QalculateWindow : public QMainWindow {
 		void exportCSV();
 		void editKeyboardShortcuts();
 		void editPreferences();
+		void openSettingsFolder();
 		void onDatasetsChanged();
 		void onFunctionsChanged();
 		void insertProperty(DataObject*, DataProperty*);
 		void openDatasets();
 		void openFunctions();
+		void resetUnitsAndVariablesMenus();
 		void openUnits();
 		void openVariables();
 		void applyFunction(MathFunction*);
